/******************************************************************************
 *
 * Copyright 2013 Altera Corporation. All Rights Reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * 
 * 3. The name of the author may not be used to endorse or promote products
 * derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 * 
 ******************************************************************************/
/********************************************************************************//*!
 * @file hwlib.c 
 * 
 * @note  'SoC EDS'exampleɎ^ꂽt@CKvȏcăTvp̌ĂяoVKǉB 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/06  ---  #6192  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include "alt_interrupt.h"

/// extern from sample_gpio.c ///
extern void sample_gpio_iointerrupt(void);
extern void sample_gpio_iopolling(void);
extern void sample_gpio_iconfig(void);
extern void sample_gpio_config(void);
extern void sample_gpio_utility(void);
/////

#ifndef MIN
#define MIN(a, b) ((a) < (b) ? (a) : (b))
#endif

/////

//
// Initializes and enables the interrupt controller.
//
ALT_STATUS_CODE socfpga_int_start(void)
{
    ALT_STATUS_CODE status = ALT_E_SUCCESS;

    //
    // Initialize the global and CPU interrupt items
    //

    if (status == ALT_E_SUCCESS)
    {
        status = alt_int_global_init();
        if (status != ALT_E_SUCCESS)
        {
            printf("ERROR: alt_int_global_init() failed, %d.\n", (int)status);
        }
    }

    if (status == ALT_E_SUCCESS)
    {
        status = alt_int_cpu_init();
        if (status != ALT_E_SUCCESS)
        {
            printf("ERROR: alt_int_cpu_init() failed, %d.\n", (int)status);
        }
    }

    //
    // Enable the CPU and global interrupt
    //

    if (status == ALT_E_SUCCESS)
    {
        status = alt_int_cpu_enable();
        if (status != ALT_E_SUCCESS)
        {
            printf("ERROR: alt_int_cpu_enable() failed, %d.\n", (int)status);
        }
    }

    if (status == ALT_E_SUCCESS)
    {
        status = alt_int_global_enable();
        if (status != ALT_E_SUCCESS)
        {
            printf("ERROR: alt_int_global_enable() failed, %d.\n", (int)status);
        }
    }

    return status;
}

void socfpga_int_stop(void)
{
    //
    // Disable the global, CPU, and distributor interrupt
    //

    if (alt_int_global_disable() != ALT_E_SUCCESS)
    {
        printf("WARN: alt_int_global_disable() return non-SUCCESS.");
    }

    if (alt_int_cpu_disable() != ALT_E_SUCCESS)
    {
        printf("WARN: alt_int_cpu_disable() return non-SUCCESS.");
    }

    //
    // Uninitialize the CPU and global data structures.
    //

    if (alt_int_cpu_uninit() != ALT_E_SUCCESS)
    {
        printf("WARN: alt_int_cpu_uninit() return non-SUCCESS.\n");
    }

    if (alt_int_global_uninit() != ALT_E_SUCCESS)
    {
        printf("WARN: alt_int_global_uninit() return non-SUCCESS.\n");
    }
}

int main(int argc, char** argv)
{
    ALT_STATUS_CODE status = ALT_E_SUCCESS;

    //
    // Start the interrupt system
    //
    if (status == ALT_E_SUCCESS)
    {
        status = socfpga_int_start();
    }

    //// Sample Function Call!!! |---->
    sample_gpio_utility();
    sample_gpio_config();
    sample_gpio_iconfig();
    sample_gpio_iointerrupt();
    sample_gpio_iopolling();
    //// Sample Function Call!!! <----|

    //
    // Stop the interrupt system
    //
    socfpga_int_stop();

    printf("Finished running the sample !!!\n");
    if (status == ALT_E_SUCCESS)
    {
        return 0;
    }
    else
    {
        return 1;
    }
}
