/********************************************************************************//*!
 * @file  sample_globaltimer.c 
 * @brief  To use HWLib for the Global Timer example program. 
 * 
 * @details  Run the API category below.
 *           - The Global Timer Manager API  
 * 
 * @note  nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/07  ---  #6203  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 * @date 2014/08/18  ---  -----  v14.0 (AA̕\ǉ). 
 * 
 * @attention 
 * Copyright (C) 2013-2014 ALTIMA Corp./ELSENA,Inc. All Rights Reserved.\n
 *   This software is licensed "AS IS". 
 *   Please perform use of this software by a user's own responsibility and expense.
 *   It cannot guarantee in the maker side about the damage which occurred by the ab-
 *   ility not to use or use this software, and all damage that occurred secondarily.
 **//*******************************************************************************/

/***********************************************************************************
 *  includes 
 ***********************************************************************************/
#include <stdio.h>
#include <string.h>
#include "hwlib.h"
#include "alt_generalpurpose_io.h"
#include "alt_globaltmr.h"
#include "alt_interrupt.h"

/***********************************************************************************
 *  externs 
 ***********************************************************************************/

/***********************************************************************************
 *  proto types 
 ***********************************************************************************/
void sample_gpio_config_helioboard(void);
void sample_globaltimer_setting_gic(void);
void sample_globaltimer_callback(uint32_t icciar, void *context);
void sample_globaltimer_print_get_result(void);
void sample_globaltimer_alldisable(void);
void sample_globaltimer_allenable(void);
void sample_globaltimer_paraminit(void);
void sample_globaltimer_test(void);
void sample_test_print_usage(void);

/***********************************************************************************
 *  definitions (define, enum, typedef, etc..) 
 ***********************************************************************************/
/** memo: port asign ***************/
/* Signal Name <--> HPS Port Name  */
/* USER_DIPSW_HPS0 <--> HPS_GPI13  */
/* USER_DIPSW_HPS1 <--> HPS_GPI12  */
/* USER_DIPSW_HPS2 <--> HPS_GPI11  */
/* USER_DIPSW_HPS3 <--> HPS_GPI10  */
/* USER_PB_HPS0 <--> HPS_GPI9      */
/* USER_PB_HPS1 <--> HPS_GPI8      */
/* USER_PB_HPS2 <--> HPS_GPI7      */
/* USER_PB_HPS3 <--> HPS_GPI6      */
/* USER_HPS_GPIO61 <--> HPS_GPIO61 */
/***********************************/
#define ALT_GPIO_PORTBIT_DIPS1		(ALT_GPIO_BIT26)
#define ALT_GPIO_PORTBIT_DIPS2		(ALT_GPIO_BIT25)
#define ALT_GPIO_PORTBIT_DIPS3		(ALT_GPIO_BIT24)
#define ALT_GPIO_PORTBIT_DIPS4		(ALT_GPIO_BIT23)
#define ALT_GPIO_PORTBIT_DIPSWALL	((ALT_GPIO_BIT26)|(ALT_GPIO_BIT25)|(ALT_GPIO_BIT24)|(ALT_GPIO_BIT23))
#define ALT_GPIO_LSBBIT_DIPSWS		(23)
#define ALT_GPIO_PORTBIT_PUSH2		(ALT_GPIO_BIT22)
#define ALT_GPIO_PORTBIT_PUSH0		(ALT_GPIO_BIT21)
#define ALT_GPIO_PORTBIT_PUSH3		(ALT_GPIO_BIT20)
#define ALT_GPIO_PORTBIT_PUSH1		(ALT_GPIO_BIT19)
#define ALT_GPIO_PORTBIT_PUSHSWALL	((ALT_GPIO_BIT22)|(ALT_GPIO_BIT21)|(ALT_GPIO_BIT20)|(ALT_GPIO_BIT19))
#define ALT_GPIO_PORTBIT_SWALL		((ALT_GPIO_PORTBIT_DIPSWALL)|(ALT_GPIO_PORTBIT_PUSHSWALL))

/***********************************************************************************
 *  variables 
 ***********************************************************************************/

/***********************************************************************************
 *  functions 
 ***********************************************************************************/

/********************************************************************************//*!
 * @fn  void sample_gpio_config_helioboard(void) 
 * @brief  Configure the Helio Board GPIO settings. 
 * 
 * @details  Performs the following processing. 
 *           - Gets the contents of the current port settings to gpio_config_array. 
 *           - Global timer counter value of the measurement at the start
 *           - Setting DIPSW and PUSHSW via bit index configuration func. 
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/07  ---  #6203  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
void sample_gpio_config_helioboard(void)
{
	ALT_STATUS_CODE result_code;
	ALT_GPIO_CONFIG_RECORD_t gpio_config_array[10];
	ALT_GPIO_CONFIG_RECORD_t* p_gpiocfg;

	// Setting OUT-PORTiGPIO61j via normal configuration func.
	/****************************************************************************************/
	/* Usage: ALT_STATUS_CODE alt_gpio_port_config(	ALT_GPIO_PORT_t gpio_pid,
	 * 												uint32_t mask,
	 * 												ALT_GPIO_PIN_DIR_t dir,
	 * 												ALT_GPIO_PIN_TYPE_t type,
	 * 												ALT_GPIO_PIN_POL_t pol,
	 * 												ALT_GPIO_PIN_DEBOUNCE_t debounc,
	 * 												uint32_t data); 						*/
	/****************************************************************************************/
	result_code = alt_gpio_port_config(ALT_GPIO_PORTC, ALT_GPIO_BIT3,
			ALT_GPIO_PIN_OUTPUT, ALT_GPIO_PIN_LEVEL_TRIG_INT, ALT_GPIO_PIN_ACTIVE_HIGH, ALT_GPIO_PIN_NODEBOUNCE,
			ALT_GPIO_BIT3);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_gpio_port_config() result=%d\n", (int)result_code);
	}

	// Gets the contents of the current port settings to gpio_config_array.
	/****************************************************************************************/
	/* Usage: ALT_STATUS_CODE alt_gpio_group_config_get(ALT_GPIO_CONFIG_RECORD_t* config_array, uint32_t len);	*/
	/****************************************************************************************/
	gpio_config_array[0].signal_number = ALT_GPIO_1BIT_61;
	gpio_config_array[1].signal_number = ALT_HLGPI_6;
	gpio_config_array[2].signal_number = ALT_HLGPI_7;
	gpio_config_array[3].signal_number = ALT_HLGPI_8;
	gpio_config_array[4].signal_number = ALT_HLGPI_9;
	gpio_config_array[5].signal_number = ALT_HLGPI_10;
	gpio_config_array[6].signal_number = ALT_HLGPI_11;
	gpio_config_array[7].signal_number = ALT_HLGPI_12;
	gpio_config_array[8].signal_number = ALT_HLGPI_13;
	gpio_config_array[9].signal_number = ALT_END_OF_GPIO_SIGNALS;
	result_code = alt_gpio_group_config_get(gpio_config_array, 10);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_gpio_group_config_get() result=%d\n", (int)result_code);
	}

	// Setting DIPSW and PUSHSW via bit index configuration func.
	/****************************************************************************************/
	/* Usage: ALT_STATUS_CODE alt_gpio_group_config(ALT_GPIO_CONFIG_RECORD_t* config_array, uint32_t len); */
	/****************************************************************************************/
	for(int i=1; i<10; i++)
	{
		p_gpiocfg = &gpio_config_array[i];
		p_gpiocfg->direction = ALT_GPIO_PIN_INPUT;
		p_gpiocfg->type = ALT_GPIO_PIN_LEVEL_TRIG_INT;
		p_gpiocfg->polarity = ALT_GPIO_PIN_ACTIVE_HIGH;
		p_gpiocfg->debounce = ALT_GPIO_PIN_NODEBOUNCE;
		p_gpiocfg->data = ALT_GPIO_PIN_DATAZERO;
	}
	result_code = alt_gpio_group_config(gpio_config_array, 10);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_gpio_group_config() result=%d\n", (int)result_code);
	}

	return;
}

/********************************************************************************//*!
 * @fn  void sample_globaltimer_setting_gic(void) 
 * @brief  Setting of GIC (Global Timer interrupt).
 * 
 * @details    In order to receive a Global Timer interrupt setting of GIC.
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/07  ---  #6203  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
void sample_globaltimer_setting_gic(void)
{
	ALT_STATUS_CODE result_code;
	uint32_t ui32temp = 0;
	ALT_INT_TRIGGER_t trigger_type = ALT_INT_TRIGGER_NA;

	// Print initial value (GIC: GPIO2)
	/****************************************************************************************/
	/* Usage: ALT_STATUS_CODE alt_int_dist_is_secure(ALT_INT_INTERRUPT_t int_id);	*/
	/* Usage: ALT_STATUS_CODE alt_int_dist_is_enabled(ALT_INT_INTERRUPT_t int_id);	*/
	/* Usage: ALT_STATUS_CODE alt_int_dist_is_pending(ALT_INT_INTERRUPT_t int_id);	*/
	/* Usage: ALT_STATUS_CODE alt_int_dist_is_active(ALT_INT_INTERRUPT_t int_id);	*/
	/* Usage: ALT_STATUS_CODE alt_int_dist_priority_get(ALT_INT_INTERRUPT_t int_id, uint32_t* priority);	*/
	/* Usage: ALT_STATUS_CODE alt_int_dist_target_get(ALT_INT_INTERRUPT_t int_id, alt_int_cpu_target_t* target);	*/
	/* Usage: ALT_STATUS_CODE alt_int_dist_trigger_get(ALT_INT_INTERRUPT_t int_id, ALT_INT_TRIGGER_t* trigger_type);	*/
	/****************************************************************************************/
	printf("-- HWLib test : alt_int_dist_is_secure() --\n");
	printf("arg1=ALT_INT_INTERRUPT_PPI_TIMER_GLOBAL --> result=%d\n", (int)alt_int_dist_is_secure(ALT_INT_INTERRUPT_PPI_TIMER_GLOBAL));
	printf("\n");
	printf("-- HWLib test : alt_int_dist_is_enabled() --\n");
	printf("arg1=ALT_INT_INTERRUPT_PPI_TIMER_GLOBAL --> result=%d\n", (int)alt_int_dist_is_enabled(ALT_INT_INTERRUPT_PPI_TIMER_GLOBAL));
	printf("\n");
	printf("-- HWLib test : alt_int_dist_is_pending() --\n");
	printf("arg1=ALT_INT_INTERRUPT_PPI_TIMER_GLOBAL --> result=%d\n", (int)alt_int_dist_is_pending(ALT_INT_INTERRUPT_PPI_TIMER_GLOBAL));
	printf("\n");
	printf("-- HWLib test : alt_int_dist_is_active() --\n");
	printf("arg1=ALT_INT_INTERRUPT_PPI_TIMER_GLOBAL --> result=%d\n", (int)alt_int_dist_is_active(ALT_INT_INTERRUPT_PPI_TIMER_GLOBAL));
	printf("\n");
	printf("-- HWLib test : alt_int_dist_priority_get() --\n");
	printf("arg1=ALT_INT_INTERRUPT_PPI_TIMER_GLOBAL --> result=%d out=0x%08X\n", (int)alt_int_dist_priority_get(ALT_INT_INTERRUPT_PPI_TIMER_GLOBAL,&ui32temp), (int)ui32temp);
	printf("\n");
	printf("-- HWLib test : alt_int_dist_target_get() --\n");
	printf("arg1=ALT_INT_INTERRUPT_PPI_TIMER_GLOBAL --> result=%d out=0x%08X\n", (int)alt_int_dist_target_get(ALT_INT_INTERRUPT_PPI_TIMER_GLOBAL,&ui32temp), (int)ui32temp);
	printf("\n");
	printf("-- HWLib test : alt_int_dist_trigger_get() --\n");
	printf("arg1=ALT_INT_INTERRUPT_PPI_TIMER_GLOBAL --> result=%d out=%d\n", (int)alt_int_dist_trigger_get(ALT_INT_INTERRUPT_PPI_TIMER_GLOBAL,&trigger_type), (int)trigger_type);
	printf("\n");

	/* Usage: ALT_STATUS_CODE alt_int_isr_register(ALT_INT_INTERRUPT_t int_id, alt_int_callback_t callback, void* context);	*/
	/*  typedef void(* alt_int_callback_t)(uint32_t icciar, void *context) */
	printf("-- HWLib test : alt_int_isr_register() --\n");
	result_code = alt_int_isr_register(ALT_INT_INTERRUPT_PPI_TIMER_GLOBAL, sample_globaltimer_callback, NULL);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_int_isr_register() result=%d\n", (int)result_code);
	}
	else
	{
		printf("SUCCESS!!\n");
	}
	printf("\n");

	///* Usage: ALT_STATUS_CODE alt_int_dist_target_set(ALT_INT_INTERRUPT_t int_id, alt_int_cpu_target_t target);	*/
	//printf("-- HWLib test : alt_int_dist_target_set() --\n");
	//result_code = alt_int_dist_target_set(ALT_INT_INTERRUPT_PPI_TIMER_GLOBAL, 1);		// arg2=targetCPU (Only Core#0)
	//if(result_code != ALT_E_SUCCESS)
	//{
	//	printf("ERROR!!: alt_int_dist_target_set() result=%d\n", (int)result_code);
	//}
	//else
	//{
	//	printf("SUCCESS!!\n");
	//}
	//printf("\n");

	/* Usage: ALT_STATUS_CODE alt_int_dist_trigger_set(ALT_INT_INTERRUPT_t int_id, ALT_INT_TRIGGER_t trigger_type);	*/
	printf("-- HWLib test : alt_int_dist_trigger_set() --\n");
	result_code = alt_int_dist_trigger_set(ALT_INT_INTERRUPT_PPI_TIMER_GLOBAL, ALT_INT_TRIGGER_EDGE);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_int_dist_trigger_set() result=%d\n", (int)result_code);
	}
	else
	{
		printf("SUCCESS!!\n");
	}
	printf("\n");

	/* Usage: ALT_STATUS_CODE alt_int_dist_priority_set(ALT_INT_INTERRUPT_t int_id, uint32_t priority);	*/
	printf("-- HWLib test : alt_int_dist_priority_set() --\n");
	result_code = alt_int_dist_priority_set(ALT_INT_INTERRUPT_PPI_TIMER_GLOBAL, 99);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_int_dist_priority_set() result=%d\n", (int)result_code);
	}
	else
	{
		printf("SUCCESS!!\n");
	}
	printf("\n");

	/* Usage: ALT_STATUS_CODE alt_int_dist_enable(ALT_INT_INTERRUPT_t int_id);	*/
	printf("-- HWLib test : alt_int_dist_enable() --\n");
	result_code = alt_int_dist_enable(ALT_INT_INTERRUPT_PPI_TIMER_GLOBAL);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_int_dist_enable() result=%d\n", (int)result_code);
	}
	else
	{
		printf("SUCCESS!!\n");
	}
	printf("\n");

	/* Usage: ALT_STATUS_CODE alt_int_dist_secure_enable(ALT_INT_INTERRUPT_t int_id);	*/
	printf("-- HWLib test : alt_int_dist_secure_enable() --\n");
	result_code = alt_int_dist_secure_enable(ALT_INT_INTERRUPT_PPI_TIMER_GLOBAL);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_int_dist_enable() result=%d\n", (int)result_code);
	}
	else
	{
		printf("SUCCESS!!\n");
	}
	printf("\n");


	return;
}

/********************************************************************************//*!
 * @fn  void sample_globaltimer_callback(uint32_t icciar, void *context) 
 * @brief  Global Timer interrupt callback function.
 * 
 * @details  working opportunity an interrupt from the Global Timer, indicate an 
 *           interrupt, and the interrupt pending bit clear.
 * @param[in] icciar  The callback to use when the given interrupt is issued. 
 * @param[in] context  The user provided context.  
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/07  ---  #6203  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
void sample_globaltimer_callback(uint32_t icciar, void *context)
{
	ALT_STATUS_CODE result_code;

	/* Usage: ALT_STATUS_CODE alt_globaltmr_int_clear_pending(void);	*/
	result_code = alt_globaltmr_int_clear_pending();
	if(result_code != ALT_E_SUCCESS)
	{
		printf("[INTERRUPT]ERROR!!: alt_globaltmr_int_clear_pending() result=%d\n", (int)result_code);
	}

	/* Usage: ALT_STATUS_CODE alt_int_dist_pending_clear(ALT_INT_INTERRUPT_t int_id);	*/
	result_code = alt_int_dist_pending_clear(ALT_INT_INTERRUPT_PPI_TIMER_GLOBAL);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("[INTERRUPT]ERROR!!: alt_int_dist_pending_clear() result=%d\n", (int)result_code);
	}

	printf("[INTERRUPT]Global Timer Interrupt is occurred!!\n");

	return;
}


/********************************************************************************//*!
 * @fn  void sample_globaltimer_print_get_result(void) 
 * @brief  Global timer information get and display. 
 * 
 * @details  Use the global timer API and briefly displays the available information. 
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/07  ---  #6203  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
void sample_globaltimer_print_get_result(void)
{
	uint32_t ui32data[2];
	ALT_STATUS_CODE	result_code;

	// All APIs for information getting are performed. And a result is printed.
	// Counters (Register Values)
	/***********************************************************************/
	/* Usage: ALT_STATUS_CODE alt_globaltmr_get(uint32_t* highword, uint32_t* lowword);	*/
	/* Usage: uint64_t alt_globaltmr_get64(void);	*/
	/* Usage: uint32_t alt_globaltmr_counter_get_low32(void);	*/
	/* Usage: uint32_t alt_globaltmr_counter_get_hi32(void);	*/
	/* Usage: ALT_STATUS_CODE alt_globaltmr_comp_get(uint32_t* highword, uint32_t* lowword);	*/
	/* Usage: uint64_t alt_globaltmr_comp_get64(void);	*/
	/* Usage: uint32_t alt_globaltmr_prescaler_get(void);	*/
	/* Usage: uint32_t alt_globaltmr_autoinc_get(void);	*/
	/* Usage: uint32_t alt_globaltmr_maxcounter_get(void);	*/
	/***********************************************************************/
	printf("-- HWLib test : alt_globaltmr_get() --\n");
	result_code = alt_globaltmr_get(&ui32data[0], &ui32data[1]);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_globaltmr_get() result=%d\n", (int)result_code);
	} else {
		printf("result { highword=0x%08X, lowword=0x%08X }\n", (int)ui32data[0], (int)ui32data[1]);
	}
	printf("-- HWLib test : alt_globaltmr_get64() --\n");
	printf("result=0x%016llX\n", (long long int)alt_globaltmr_get64());
	printf("-- HWLib test : alt_globaltmr_counter_get_low32(), alt_globaltmr_counter_get_hi32()--\n");
	ui32data[0] = alt_globaltmr_counter_get_hi32();
	ui32data[1] = alt_globaltmr_counter_get_low32();
	printf("result { highword=0x%08X, lowword=0x%08X }\n", (int)ui32data[0], (int)ui32data[1]);
	printf("-- HWLib test : alt_globaltmr_comp_get() --\n");
	result_code = alt_globaltmr_comp_get(&ui32data[0], &ui32data[1]);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_globaltmr_comp_get() result=%d\n", (int)result_code);
	} else {
		printf("result { highword=0x%08X, lowword=0x%08X }\n", (int)ui32data[0], (int)ui32data[1]);
	}
	printf("-- HWLib test : alt_globaltmr_comp_get64() --\n");
	printf("result=0x%016llX\n", (long long int)alt_globaltmr_comp_get64());
	printf("-- HWLib test : alt_globaltmr_prescaler_get() --\n");
	printf("result=0x%08X\n", (int)alt_globaltmr_prescaler_get());
	printf("-- HWLib test : alt_globaltmr_autoinc_get() --\n");
	printf("result=0x%08X\n", (int)alt_globaltmr_autoinc_get());
	printf("-- HWLib test : alt_globaltmr_maxcounter_get() --\n");
	printf("result=0x%08X\n", (int)alt_globaltmr_maxcounter_get());

	// Statuses
	/***********************************************************************/
	/* Usage: bool alt_globaltmr_is_comp_mode(void);	*/
	/* Usage: bool alt_globaltmr_is_autoinc_mode(void);	*/
	/* Usage: bool alt_globaltmr_int_is_enabled(void);	*/
	/* Usage: bool alt_globaltmr_int_is_pending(void);	*/
	/***********************************************************************/
	printf("-- HWLib test : alt_globaltmr_is_comp_mode() --\n");
	printf("result=%d\n", (int)alt_globaltmr_is_comp_mode());
	printf("-- HWLib test : alt_globaltmr_is_autoinc_mode() --\n");
	printf("result=%d\n", (int)alt_globaltmr_is_autoinc_mode());
	printf("-- HWLib test : alt_globaltmr_int_is_enabled() --\n");
	printf("result=%d\n", (int)alt_globaltmr_int_is_enabled());
	printf("-- HWLib test : alt_globaltmr_int_is_pending() --\n");
	printf("result=%d\n", (int)alt_globaltmr_int_is_pending());
	printf("\n");

	return;
}

/********************************************************************************//*!
 * @fn  void sample_globaltimer_alldisable(void) 
 * @brief  To disable all features of the global timer.  
 * 
 * @details  Stop a global timer. Also, disable comparator mode, auto-increment 
 *           mode, interrupt notification. 
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/07  ---  #6203  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
void sample_globaltimer_alldisable(void)
{
	ALT_STATUS_CODE	result_code;

	// All Functions of Global Timer are disabled.
	/***********************************************************************/
	/* Usage: ALT_STATUS_CODE alt_globaltmr_stop(void);	*/
	/* Usage: ALT_STATUS_CODE alt_globaltmr_comp_mode_stop(void);	*/
	/* Usage: ALT_STATUS_CODE alt_globaltmr_autoinc_mode_stop(void);	*/
	/* Usage: ALT_STATUS_CODE alt_globaltmr_int_disable(void);	*/
	/* Usage: ALT_STATUS_CODE alt_globaltmr_int_clear_pending(void);	*/
	/***********************************************************************/
	printf("-- HWLib test : alt_globaltmr_stop() --\n");
	result_code = alt_globaltmr_stop();
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_globaltmr_stop() result=%d\n", (int)result_code);
	} else {
		printf("SUCCESS!!\n");
	}
	printf("-- HWLib test : alt_globaltmr_comp_mode_stop() --\n");
	result_code = alt_globaltmr_comp_mode_stop();
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_globaltmr_comp_mode_stop() result=%d\n", (int)result_code);
	} else {
		printf("SUCCESS!!\n");
	}
	printf("-- HWLib test : alt_globaltmr_autoinc_mode_stop() --\n");
	result_code = alt_globaltmr_autoinc_mode_stop();
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_globaltmr_autoinc_mode_stop() result=%d\n", (int)result_code);
	} else {
		printf("SUCCESS!!\n");
	}
	printf("-- HWLib test : alt_globaltmr_int_disable() --\n");
	result_code = alt_globaltmr_int_disable();
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_globaltmr_int_disable() result=%d\n", (int)result_code);
	} else {
		printf("SUCCESS!!\n");
	}
	printf("-- HWLib test : alt_globaltmr_int_clear_pending() --\n");
	result_code = alt_globaltmr_int_clear_pending();
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_globaltmr_int_clear_pending() result=%d\n", (int)result_code);
	} else {
		printf("SUCCESS!!\n");
	}
	printf("\n");

	return;
}

/********************************************************************************//*!
 * @fn  void sample_globaltimer_allenable(void) 
 * @brief  To enable all features of the global timer.  
 * 
 * @details  Start a global timer. Also, enable comparator mode, auto-increment 
 *           mode, interrupt notification. 
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/07  ---  #6203  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
void sample_globaltimer_allenable(void)
{
	ALT_STATUS_CODE	result_code;

	// All Functions of Global Timer are enabled.
	/***********************************************************************/
	/* Usage: bool alt_globaltmr_int_if_pending_clear(void);	*/
	/* Usage: ALT_STATUS_CODE alt_globaltmr_start(void);	*/
	/* Usage: ALT_STATUS_CODE alt_globaltmr_comp_mode_start(void);	*/
	/* Usage: ALT_STATUS_CODE alt_globaltmr_autoinc_mode_start(void);	*/
	/* Usage: ALT_STATUS_CODE alt_globaltmr_int_enable(void);	*/
	/***********************************************************************/
	printf("-- HWLib test : alt_globaltmr_int_if_pending_clear() --\n");
	printf("result=%d\n", (int)alt_globaltmr_int_if_pending_clear());
	printf("-- HWLib test : alt_globaltmr_start() --\n");
	result_code = alt_globaltmr_start();
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_globaltmr_start() result=%d\n", (int)result_code);
	} else {
		printf("SUCCESS!!\n");
	}
	printf("-- HWLib test : alt_globaltmr_comp_mode_start() --\n");
	result_code = alt_globaltmr_comp_mode_start();
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_globaltmr_comp_mode_start() result=%d\n", (int)result_code);
	} else {
		printf("SUCCESS!!\n");
	}
	printf("-- HWLib test : alt_globaltmr_autoinc_mode_start() --\n");
	result_code = alt_globaltmr_autoinc_mode_start();
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_globaltmr_autoinc_mode_start() result=%d\n", (int)result_code);
	} else {
		printf("SUCCESS!!\n");
	}
	printf("-- HWLib test : alt_globaltmr_int_enable() --\n");
	result_code = alt_globaltmr_int_enable();
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_globaltmr_int_enable() result=%d\n", (int)result_code);
	} else {
		printf("SUCCESS!!\n");
	}
	printf("\n");

	return;
}

/********************************************************************************//*!
 * @fn  void sample_globaltimer_paraminit(void) 
 * @brief  Initialize parameters for global timer.
 * 
 * @details    Set the initial value of the prescaler, AutoIncrement, comparator 
 *             as a global timer parameter initialization.
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/07  ---  #6203  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
void sample_globaltimer_paraminit(void)
{
	ALT_STATUS_CODE	result_code;

	// Parameter setting for the global timer.
	/***********************************************************************/
	/* Usage: ALT_STATUS_CODE alt_globaltmr_prescaler_set(uint32_t val);	*/
	/*			val ... Valid range is 0-255. Actual clock divisor ratio is this number plus one. */
	/***********************************************************************/
	printf("-- HWLib test : alt_globaltmr_prescaler_set() --\n");
	//result_code = alt_globaltmr_prescaler_set(255);	// Set the maximum value(255).
	result_code = alt_globaltmr_prescaler_set(0);	// Set the minimum value(0).
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_globaltmr_prescaler_set() result=%d\n", (int)result_code);
	} else {
		printf("SUCCESS!!\n");
	}
	printf("\n");
	/***********************************************************************/
	/* Usage: ALT_STATUS_CODE alt_globaltmr_autoinc_set(uint32_t inc);	*/
	/***********************************************************************/
	printf("-- HWLib test : alt_globaltmr_autoinc_set() --\n");
	result_code = alt_globaltmr_autoinc_set(0x40000000);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_globaltmr_autoinc_set() result=%d\n", (int)result_code);
	} else {
		printf("SUCCESS!!\n");
	}
	printf("\n");
	/***********************************************************************/
	/* Usage: ALT_STATUS_CODE alt_globaltmr_comp_set64(uint64_t compval);	*/
	/***********************************************************************/
	printf("-- HWLib test : alt_globaltmr_comp_set64() --\n");
	result_code = alt_globaltmr_comp_set64((uint64_t)(alt_globaltmr_comp_get64() + 0x0000000040000000ll));
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_globaltmr_comp_set64() result=%d\n", (int)result_code);
	} else {
		printf("SUCCESS!!\n");
	}
	printf("\n");

	return;
}

/********************************************************************************//*!
 * @fn  void sample_globaltimer_test(void) 
 * @brief  Main routine of the sample program. 
 * 
 * @details  Infinite loop structure works. Make the detection of various types 
 *           of switches in the loop, it performs in various testing processes.
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/07  ---  #6203  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
void sample_globaltimer_test(void)
{
	uint32_t ui32high, ui32low;
	uint32_t ui32lowbk = 0;
	uint32_t ui32hcmp, ui32lcmp;
	uint32_t ui32hcmpbk, ui32lcmpbk;
	uint32_t ui32portc = 0;
	uint32_t ui32portcbk = 0;
	uint32_t ui32portc_onchg = 0;
	uint32_t ui32prescaler;
	ALT_STATUS_CODE	result_code;

	result_code = alt_globaltmr_comp_get(&ui32hcmpbk, &ui32lcmpbk);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_globaltmr_comp_get() result=%d\n", (int)result_code);
	}

	printf("==== Start While(1) loop process!!! (Exit PUSHSW0(SW8) becomes ON.) ====\n");
	sample_test_print_usage();
	while(1)
	{

		// ---- Check the DIPSW and PUSHSW. ----
		ui32portc = alt_gpio_port_data_read(ALT_GPIO_PORTC, ALT_GPIO_PORTBIT_SWALL);
		ui32portc = ~ui32portc & ALT_GPIO_PORTBIT_SWALL;	// bit inversion and mask ({ON=0,OFF=1}->{ON=1,OFF=0})
		ui32portc_onchg |= (ui32portc ^ ui32portcbk) & ui32portc;

		// ---- Check the Global Timer Registers. ----
		result_code = alt_globaltmr_get(&ui32high, &ui32low);
		if(result_code != ALT_E_SUCCESS){
			printf("ERROR!!: alt_globaltmr_get() result=%d\n", (int)result_code);
		}

		// PUSHSW HPS 0
		if(ui32portc_onchg & ALT_GPIO_PORTBIT_PUSH0)
		{
			ui32portc_onchg &= ~ALT_GPIO_PORTBIT_PUSH0;
			
			break;	// Exit PUSHSW0(SW8) becomes ON.
		}
		// PUSHSW HPS 1
		if(ui32portc_onchg & ALT_GPIO_PORTBIT_PUSH1)
		{
			ui32portc_onchg &= ~ALT_GPIO_PORTBIT_PUSH1;
			
			result_code = alt_globaltmr_comp_get(&ui32hcmpbk, &ui32lcmpbk);
			if(result_code != ALT_E_SUCCESS){
				printf("ERROR!!: alt_globaltmr_comp_get() result=%d\n", (int)result_code);
			} else {
				result_code = alt_globaltmr_comp_set((ui32high + 1), ui32lowbk);
				if(result_code != ALT_E_SUCCESS){
					printf("ERROR!!: alt_globaltmr_comp_set() result=%d\n", (int)result_code);
				} else {
					result_code = alt_globaltmr_comp_get(&ui32hcmp, &ui32lcmp);
					if(result_code != ALT_E_SUCCESS){
						printf("ERROR!!: alt_globaltmr_comp_get() result=%d\n", (int)result_code);
					} else {
						printf("* Change Comparator Val:  0x%08X:%08X --> 0x%08X:%08X\n", 
							(int)ui32hcmpbk, (int)ui32lcmpbk, (int)ui32hcmp, (int)ui32lcmp);
					}
				}
			}
		}
		// PUSHSW HPS 2
		if(ui32portc_onchg & ALT_GPIO_PORTBIT_PUSH2)
		{
			ui32portc_onchg &= ~ALT_GPIO_PORTBIT_PUSH2;
			
			result_code = alt_globaltmr_comp_get(&ui32hcmp, &ui32lcmp);
			if(result_code != ALT_E_SUCCESS){
				printf("ERROR!!: alt_globaltmr_comp_get() result=%d\n", (int)result_code);
			} else {
				printf("* Display Comparator Val: 0x%08X:%08X\n", (int)ui32hcmp, (int)ui32lcmp);
			}
		}
		// PUSHSW HPS 3
		if(ui32portc & ALT_GPIO_PORTBIT_PUSH3)
		{
			ui32portc_onchg &= ~ALT_GPIO_PORTBIT_PUSH3;
			
			printf("GlobalTimer Counter = 0x%08X:%08X\n", (int)ui32high, (int)ui32low);
		} else
		{
			// Global Timer Counter Value
			if((ui32low ^ ui32lowbk) & 0xFC000000)
			{
				printf("GlobalTimer Counter = 0x%08X:%08X\n", (int)ui32high, (int)ui32low);
			}
		}

		// DIPSW HPS 1-4
		if((ui32portcbk ^ ui32portc) & ALT_GPIO_PORTBIT_DIPSWALL)
		{
			/* Usage: ALT_STATUS_CODE alt_globaltmr_prescaler_set(uint32_t val);	*/
			/*			val ... Valid range is 0-255. Actual clock divisor ratio is this number plus one. */
			ui32prescaler = (uint32_t)((ui32portc & ALT_GPIO_PORTBIT_DIPSWALL) >> ALT_GPIO_LSBBIT_DIPSWS);
			result_code = alt_globaltmr_prescaler_set(ui32prescaler);
			if(result_code != ALT_E_SUCCESS){
				printf("ERROR!!: alt_globaltmr_prescaler_set() result=%d\n", (int)result_code);
			} else {
				printf("Change Prescaler Setting: alt_globaltmr_prescaler_set(%d)\n", (int)ui32prescaler);
			}
		}

		ui32portcbk = ui32portc;
		ui32lowbk = ui32low;
	}
	printf("==== End While(1) loop process. (Exit PUSHSW0(SW8) becomes ON.) ====\n");

	return;
}

/********************************************************************************//*!
 * @fn  void sample_test_print_usage(void) 
 * @brief  Display usage of the sample program. 
 * 
 * @details  Following information displays on the console.
 *            - Overview of HPS for push button functions (or operations)
 *            - Overview of HPS for DIP switch functions (or operations)
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/08/18  ---  #xxxxx  VK쐬. 
 **//*******************************************************************************/
void sample_test_print_usage(void)
{
	printf("+------- Usage ( Function of HPS-PUSHSW and HPS-DIPSW )  -------+\n");
	printf("+ PUSHSW HPS 0(SW8) .... Exit Test loop!!!\n");
	printf("+ PUSHSW HPS 1(SW10) ... Update the Comparator value to delay the timer interrupt.\n");
	printf("+ PUSHSW HPS 2(SW7) .... Display the current Comparator value.\n");
	printf("+ PUSHSW HPS 3(SW9) .... Display the Global Timer counter.\n");
	printf("                         (Display continuously during push-down)\n");
	printf("+ DIPSW HPS(SW14) 1-4 .. Change the timer freqency dynamically.\n"); 
	printf("                         4-bit value set to prescaler intact.\n");
	printf("+---------------------------------------------------------------+\n\n");
	
#if 0
	@[vňȉ̏sB
	@ - TimerJE^l̉26bitڈȏi32bitbit#31:26j
	@@ ω閈ɃJE^lR\[óB
	@@ HPSpPUSHSW3ĂԂ͖ɃR\[óB
	@@ (TimerJE^̐i݋̊mFp)
	
	@ - HPSpPUSHSW1̉_@Comparator̃WX^l(64bit)
	@@ ύXB32bit"JE^̏32bitl + 1"A32bit
	@@  TimerJE^̉32bitl ɐݒ肵AύX̒lR\[óB
	@@ (Comparator̓mFpComparator̃WX^li߂)
	
	@ - HPSpPUSHSW2̉_@Comparator̃WX^l(64bit)
	@@ R\[óB(AutoIncrement@\̊mFp)
	
	@ - HPSpDIPSW1-4̕ύX_@ŁAPrescaler̐ݒlύXB
	@@ DIPSW1MSBADIPSW4LSBƂ4bit̒l̂܂
	@@ Prescalerɐݒ肵AݒeR\[óB
	@@ (Prescaler̐ݒlɘAāATimerJE^̓쑬x
	@@ ω鎖mFł܂)

	@ - HPSpPUSHSW0̉_@Ŗ[vI܂B
	@@ (eXgvOI)
#endif
	
	return;
}

// All APIs
/***********************************************************************/
/* Usage: ALT_STATUS_CODE alt_globaltmr_stop(void);	*/
/* Usage: ALT_STATUS_CODE alt_globaltmr_start(void);	*/
/* Usage: ALT_STATUS_CODE alt_globaltmr_get(uint32_t* highword, uint32_t* lowword);	*/
/* Usage: uint64_t alt_globaltmr_get64(void);	*/
/* Usage: uint32_t alt_globaltmr_counter_get_low32(void);	*/
/* Usage: uint32_t alt_globaltmr_counter_get_hi32(void);	*/
/* Usage: ALT_STATUS_CODE alt_globaltmr_comp_set(uint32_t highword, uint32_t loword);	*/
/* Usage: ALT_STATUS_CODE alt_globaltmr_comp_set64(uint64_t compval);	*/
/* Usage: ALT_STATUS_CODE alt_globaltmr_comp_get(uint32_t* highword, uint32_t* lowword);	*/
/* Usage: uint64_t alt_globaltmr_comp_get64(void);	*/
/* Usage: ALT_STATUS_CODE alt_globaltmr_comp_mode_start(void);	*/
/* Usage: ALT_STATUS_CODE alt_globaltmr_comp_mode_stop(void);	*/
/* Usage: bool alt_globaltmr_is_comp_mode(void);	*/
/* Usage: uint32_t alt_globaltmr_prescaler_get(void);	*/
/* Usage: ALT_STATUS_CODE alt_globaltmr_prescaler_set(uint32_t val);	*/
/* Usage: ALT_STATUS_CODE alt_globaltmr_autoinc_set(uint32_t inc);	*/
/* Usage: uint32_t alt_globaltmr_autoinc_get(void);	*/
/* Usage: ALT_STATUS_CODE alt_globaltmr_autoinc_mode_start(void);	*/
/* Usage: ALT_STATUS_CODE alt_globaltmr_autoinc_mode_stop(void);	*/
/* Usage: bool alt_globaltmr_is_autoinc_mode(void);	*/
/* Usage: uint32_t alt_globaltmr_maxcounter_get(void);	*/
/* Usage: ALT_STATUS_CODE alt_globaltmr_int_disable(void);	*/
/* Usage: ALT_STATUS_CODE alt_globaltmr_int_enable(void);	*/
/* Usage: bool alt_globaltmr_int_is_enabled(void);	*/
/* Usage: ALT_STATUS_CODE alt_globaltmr_int_clear_pending(void);	*/
/* Usage: bool alt_globaltmr_int_is_pending(void);	*/
/* Usage: bool alt_globaltmr_int_if_pending_clear(void);	*/
/***********************************************************************/

/***********************************************************************************
 * end of file 
 ***********************************************************************************/
